//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "ExplorerButton.h"

// Copy of the auto_ptr class to save linking loads of unwanted headers

template<class X> class Auto_ptr
{
	X* p;
public:
	explicit Auto_ptr (X* p = 0) : p(p)           {}
	Auto_ptr (Auto_ptr<X>& a)    : p(a.release()) {}
	void operator= (Auto_ptr<X>& rhs) { reset(rhs.release()); }
	~Auto_ptr () { delete p; }
	X& operator*  ()        const { return *p;   }
	X* operator-> ()        const { return p;    }
	X* get        ()        const { return p;    }
	X* release    ()              { return reset(0); }
	X* reset      (X* p = 0)      { X* tmp = p; p = p; return tmp; }
};

//---------------------------------------------------------------------------

// Unit-scope variables

TGlyphCache GlyphCache;
Graphics::TBitmap* Pattern = 0;

//---------------------------------------------------------------------------

// Unit-scope procedures

TRect DrawButtonSurface(TCanvas* Canvas, const TRect& Client,
	TButtonPosition ButtonPosition, bool IsTransparent, bool SoftShading)
{
	// DrawButtonSurface - returns the remaining usable area inside the Client rect

	HDC DC = Canvas->Handle;   // Reduce calls to GetHandle
	TRect Result, R = Client;

	UINT Adjustment = IsTransparent ? 0: BF_MIDDLE;

	if (ButtonPosition == bpDown)
	{	// DrawEdge is faster than Polyline
		DrawEdge(DC, &(RECT)R, SoftShading ? BDR_SUNKENOUTER: BDR_SUNKENINNER, BF_TOPLEFT + Adjustment); // btnhilite or black
		DrawEdge(DC, &(RECT)R, BDR_SUNKENOUTER, BF_BOTTOMRIGHT);       // btnhilite
		if (!SoftShading)
		{
			R.Bottom--;
			R.Right--;
			R.Top++;
			R.Left++;
			DrawEdge(DC, &(RECT)R, BDR_SUNKENOUTER, BF_TOPLEFT); // btnshadow
		}
		Result = Rect(Client.Left + 2, Client.Top + 2,
					Client.Right - 1, Client.Bottom - 1);
	}
	else if (ButtonPosition == bpUp)
	{
		DrawEdge(DC, &(RECT)R, SoftShading ? BDR_RAISEDINNER: BDR_RAISEDOUTER, BF_BOTTOMRIGHT + Adjustment); // black
		DrawEdge(DC, &(RECT)R, BDR_RAISEDINNER, BF_TOPLEFT);           // btnhilite
		if (!SoftShading)
		{
			R.Bottom--;
			R.Right--;
			R.Top++;
			R.Left++;
			DrawEdge(DC, &(RECT)R, BDR_RAISEDINNER, BF_BOTTOMRIGHT); // btnshadow
		}
		Result = Rect(Client.Left + 1, Client.Top + 1,
					Client.Right - 2, Client.Bottom - 2);
	}
	else // Flat
	{
		DrawEdge(DC, &(RECT)R, 0, Adjustment);
		Result = Rect(Client.Left + 1, Client.Top + 1,
					Client.Right - 2, Client.Bottom - 2);
	}

	return Result;
}

void CreateBrushPattern()
{
	if (Pattern) return;
	Pattern = new Graphics::TBitmap();
	Pattern->Width = Pattern->Height = 8;

	Pattern->Canvas->Brush->Style = bsSolid;
	Pattern->Canvas->Brush->Color = clBtnFace;
	Pattern->Canvas->FillRect(Rect(0, 0, Pattern->Width, Pattern->Height));
	for (int Y = 0; Y <= 7; Y++)
		for (int X = 0; X <= 7; X++)
			if (Y % 2 == X % 2)	 // toggles between even/odd pixels on even/odd rows
				Pattern->Canvas->Pixels[X][Y] = clWhite;
}

//---------------------------------------------------------------------------

__fastcall TGlyphList::TGlyphList(TComponent* AOwner)
:TImageList(AOwner)
{
	Width = 16;
	Height = 16;
	Used = new TBits();
}

__fastcall TGlyphList::TGlyphList(TComponent* AOwner, int AWidth, int AHeight)
:TImageList(AOwner)
{
	Width = AWidth;
	Height = AHeight;
	Used = new TBits();
}

__fastcall TGlyphList::~TGlyphList()
{
	delete Used;
}

int __fastcall TGlyphList::AllocateIndex()
{
	int Result = Used->OpenBit();
	if (Result >= Used->Size)
	{
		Result = TImageList::Add(0, 0);
		Used->Size = Result + 1;
	}
	Used->Bits[Result] = true;
	return Result;
}

int __fastcall TGlyphList::Add(Graphics::TBitmap* Image, Graphics::TBitmap* Mask)
{
	int Result = AllocateIndex();
	Replace(Result, Image, Mask);
	FCount++;
	return Result;
}

int __fastcall TGlyphList::AddMasked(Graphics::TBitmap* Image, TColor MaskColor)
{
	int Result = AllocateIndex();
	ReplaceMasked(Result, Image, MaskColor);
	FCount++;
	return Result;
}

void __fastcall TGlyphList::Delete(int Index)
{
	if (Used->Bits[Index])
	{
		FCount--;
		Used->Bits[Index] = false;
	}
}

//---------------------------------------------------------------------------

__fastcall TGlyphCache::TGlyphCache()
{
	GlyphLists = new TList();
}

__fastcall TGlyphCache::~TGlyphCache()
{
	delete GlyphLists;
}

TGlyphList* __fastcall TGlyphCache::GetList(int AWidth, int AHeight)
{
	TGlyphList* Result;

	for (int i = GlyphLists->Count - 1; i >= 0 ; i--)
	{
		Result = static_cast<TGlyphList*>(GlyphLists->Items[i]);
		if (Result->Width == AWidth && Result->Height == AHeight)
			return Result;
	}
	Result = new TGlyphList(0, AWidth, AHeight);
	GlyphLists->Add(Result);
	return Result;
}

void __fastcall TGlyphCache::ReturnList(TGlyphList* List)
{
	if (List == 0) return;
	if (List->Count == 0)
	{
		GlyphLists->Remove(List);
		delete List;
	}
}

bool __fastcall TGlyphCache::Empty()
{
	return GlyphLists->Count == 0;
}

//---------------------------------------------------------------------------

__fastcall TButtonGlyph::TButtonGlyph():
TObject()
{
	FOriginal = new Graphics::TBitmap();
	FOriginal->OnChange = GlyphChanged;
	FTransparentColor = clOlive;
	FNumGlyphs = 1;
	for (int i = 0; i <= gtLast; i++)
		FIndexs[i] = -1;
}

__fastcall TButtonGlyph::~TButtonGlyph()
{
	delete FOriginal;
	Invalidate();
}

// Invalidate() removes all the glyphs (Up, Down, Disabled etc) for this glyph
// from the glyphlist they lie in.
void __fastcall TButtonGlyph::Invalidate()
{
	for (int i = 0; i <= gtLast; i++)
	{
		if (FIndexs[i] != -1) FGlyphList->Delete(FIndexs[i]);
		FIndexs[i] = -1;
	}
	GlyphCache.ReturnList(FGlyphList);
	FGlyphList = 0;
}

void __fastcall TButtonGlyph::GlyphChanged(TObject* Sender)
{
	if (Sender == FOriginal)
	{
		FTransparentColor = FOriginal->TransparentColor;
		Invalidate();
		if (FOnChange != 0) FOnChange(this);
	}
}

void __fastcall TButtonGlyph::SetGlyph(Graphics::TBitmap* Value)
{
	Invalidate();
	FOriginal->Assign(Value);
	if (Value != 0 && Value->Height > 0)
	{
		FTransparentColor = Value->TransparentColor;
		if (Value->Width % Value->Height == 0)
		{
			TNGlyphs Glyphs = Value->Width / Value->Height;
			if (Glyphs > gtLast + 1) Glyphs = 1;
			SetNumGlyphs(Glyphs);
		}
	}
}

void __fastcall TButtonGlyph::SetNumGlyphs(TNGlyphs Value)
{
	if (Value != FNumGlyphs && Value > 0)
	{
		Invalidate();
		FNumGlyphs = Value;
	}
}

// This function returns where in the glyphlist the image for a given state
// lies. If such an image does not exist, then one is created before returning.
int __fastcall TButtonGlyph::CreateButtonGlyph(TGlyphType State)
{
	const int ROP_DSPDxax = 0x00E20746;
	int IWidth, IHeight, Result;
	TRect IRect, ORect;
	int i, j, k;
	HDC DestDC;

	if (State == gtDown && NumGlyphs <= gtDown) State = gtUp;
	Result = FIndexs[State];
	if (Result != -1) return Result;
	if (FOriginal->Width == 0 || FOriginal->Height == 0) return Result;

	IWidth = FOriginal->Width / FNumGlyphs;
	IHeight = FOriginal->Height;
	if (FGlyphList == 0) FGlyphList = GlyphCache.GetList(IWidth, IHeight);

	Auto_ptr<Graphics::TBitmap> TmpImage(new Graphics::TBitmap());
	Auto_ptr<Graphics::TBitmap> MonoBmp(new Graphics::TBitmap());
	TmpImage->Width = MonoBmp->Width = IWidth;
	TmpImage->Height = MonoBmp->Height = IHeight;
	IRect = Rect(0, 0, IWidth, IHeight);
	TmpImage->Canvas->Brush->Color = clBtnFace;
	i = State;
	if (i >= NumGlyphs) i = gtUp;
	ORect = Rect(i * IWidth, 0, (i + 1) * IWidth, IHeight);
	switch (State)
	{
	case gtUp: case gtDown: case gtAsleep:
		TmpImage->Canvas->CopyRect(IRect, FOriginal->Canvas, ORect);
		// The commented code should replace the loops, but for some reason I can't
		// get AddMasked to work as documented by Microsoft. So I do it myself.
//		FIndexs[State] = FGlyphList->AddMasked(TmpImage.get(), FTransparentColor);
		for (j = 0; j < IWidth; j++)
			for (k = 0; k < IHeight; k++)
				if (TmpImage->Canvas->Pixels[j][k] == (FTransparentColor & ~0x02000000))
				{
					MonoBmp->Canvas->Pixels[j][k] = clWhite;
					TmpImage->Canvas->Pixels[j][k] = clBlack;
				}
				else
					MonoBmp->Canvas->Pixels[j][k] = clBlack;
		FIndexs[State] = FGlyphList->Add(TmpImage.get(), MonoBmp.get());
		break;
	case gtExclusive:
		TmpImage->Canvas->CopyRect(IRect, FOriginal->Canvas, ORect);
		FIndexs[State] = FGlyphList->AddMasked(TmpImage.get(), FTransparentColor);
		break;
	case gtDisabled:
		if (NumGlyphs > 1)
		{
			// Change white & gray to clBtnHighlight and clBtnShadow
			TmpImage->Canvas->CopyRect(IRect, FOriginal->Canvas, ORect);
			MonoBmp->Monochrome = true;

			// Convert white to clBtnHighlight
			FOriginal->Canvas->Brush->Color = clWhite;
			MonoBmp->Canvas->CopyRect(IRect, FOriginal->Canvas, ORect);
			TmpImage->Canvas->Brush->Color = clBtnHighlight;
			DestDC = TmpImage->Canvas->Handle;
			SetTextColor(DestDC, clBlack);
			SetBkColor(DestDC, clWhite);
			BitBlt(DestDC, 0, 0, IWidth, IHeight,
					MonoBmp->Canvas->Handle, 0, 0, ROP_DSPDxax);

			// Convert gray to clBtnShadow
			FOriginal->Canvas->Brush->Color = clGray;
			MonoBmp->Canvas->CopyRect(IRect, FOriginal->Canvas, ORect);
			TmpImage->Canvas->Brush->Color = clBtnShadow;
			DestDC = TmpImage->Canvas->Handle;
			SetTextColor(DestDC, clBlack);
			SetBkColor(DestDC, clWhite);
			BitBlt(DestDC, 0, 0, IWidth, IHeight,
					MonoBmp->Canvas->Handle, 0, 0, ROP_DSPDxax);

			// Convert transparent color to clBtnFace
			FOriginal->Canvas->Brush->Color = static_cast<TColor>(ColorToRGB(FTransparentColor));
			MonoBmp->Canvas->CopyRect(IRect, FOriginal->Canvas, ORect);
			TmpImage->Canvas->Brush->Color = clBtnFace;
			DestDC = TmpImage->Canvas->Handle;
			SetTextColor(DestDC, clBlack);
			SetBkColor(DestDC, clWhite);
			BitBlt(DestDC, 0, 0, IWidth, IHeight,
					MonoBmp->Canvas->Handle, 0, 0, ROP_DSPDxax);
		}
		else
		{
			// Create a disabled version
			MonoBmp->Assign(FOriginal);
			MonoBmp->Canvas->Brush->Color = clBlack;
			if (MonoBmp->Monochrome)
			{
				MonoBmp->Canvas->Font->Color = clWhite;
				MonoBmp->Monochrome = false;
				MonoBmp->Canvas->Brush->Color = clWhite;
			}
			MonoBmp->Monochrome = true;

			TmpImage->Canvas->Brush->Color = clBtnFace;
			TmpImage->Canvas->FillRect(IRect);
			TmpImage->Canvas->Brush->Color = clBtnHighlight;
			SetTextColor(TmpImage->Canvas->Handle, clBlack);
			SetBkColor(TmpImage->Canvas->Handle, clWhite);
			BitBlt(TmpImage->Canvas->Handle, 1, 1, IWidth, IHeight,
					MonoBmp->Canvas->Handle, 0, 0, ROP_DSPDxax);
			TmpImage->Canvas->Brush->Color = clBtnShadow;
			SetTextColor(TmpImage->Canvas->Handle, clBlack);
			SetBkColor(TmpImage->Canvas->Handle, clWhite);
			BitBlt(TmpImage->Canvas->Handle, 0, 0, IWidth, IHeight,
					MonoBmp->Canvas->Handle, 0, 0, ROP_DSPDxax);
		}
		FIndexs[State] = FGlyphList->Add(TmpImage.get(), 0);
		break;
	}

	Result = FIndexs[State];
	FOriginal->Dormant();
	return Result;
}

void __fastcall TButtonGlyph::DrawPockMark(TCanvas* Canvas, int X, int Y, bool Horizontal,
	bool Disabled)
{
	bool nPassesLeft = 1 + Disabled;

	for (;nPassesLeft; X--, Y--)
	{
		nPassesLeft--;
		if (nPassesLeft) X++, Y++;
		TColor Color = nPassesLeft ? clWhite: (Disabled ? clDkGray: clBlack);
		if (Horizontal)
		{
			for (int i = 0; i < 4; i++)
				for (int j = i; j < 7 - i; j++)
					Canvas->Pixels[X + i + 3][Y + j + 2] = Color;
		}
		else
		{
			for (int i = 0; i < 4; i++)
				for (int j = i; j < 7 - i; j++)
					Canvas->Pixels[X + j + 3][Y + i + 2] = Color;
		}
	}
}

void __fastcall TButtonGlyph::DrawButtonGlyph(TCanvas* Canvas, int X, int Y,
			TGlyphType State)
{
	if (FOriginal == 0) return;
	if (FOriginal->Width == 0 || FOriginal->Height == 0) return;

	int Index = CreateButtonGlyph(State);

	FGlyphList->Draw(Canvas, X, Y, Index);
}

void __fastcall TButtonGlyph::DrawButtonText(TCanvas* Canvas,
		const AnsiString& Caption, TRect* TextBounds, TGlyphType State, bool WordWrap)
{
	Canvas->Brush->Style = bsClear;
	UINT uFormat = DT_CENTER + DT_VCENTER + (WordWrap ? DT_WORDBREAK: DT_SINGLELINE);
	if (State == gtDisabled)
	{	// Draw disabled-looking text
		OffsetRect((RECT *)TextBounds, 1, 1);
		Canvas->Font->Color = clWhite;
		DrawText(Canvas->Handle, Caption.c_str(), Caption.Length(), (RECT *)TextBounds, uFormat);
		OffsetRect((RECT *)TextBounds, -1, -1);
		Canvas->Font->Color = clDkGray;
		DrawText(Canvas->Handle, Caption.c_str(), Caption.Length(), (RECT *)TextBounds, uFormat);
	}
	else
	{
		DrawText(Canvas->Handle, Caption.c_str(), Caption.Length(), (RECT *)TextBounds, uFormat);
	}
}

const int PockHorzSpacing = 3;
const int PockVertSpacing = 2;

void __fastcall TButtonGlyph::CalcButtonLayout(TCanvas* Canvas, const TRect* Client,
		const AnsiString& Caption, TButtonLayout Layout, int Margin, int Spacing,
		bool WordWrap, bool PopupArrow, bool ArrowHorz, TPoint* GlyphPos, TPoint* PockPos, TRect* TextBounds)
{
	TPoint TextPos;
	TPoint ClientSize, GlyphSize, TextSize, PockSize;
	TPoint TotalSize;

	// calculate the item sizes
	ClientSize = Point(Client->Right - Client->Left, Client->Bottom -
			Client->Top);

	if (FOriginal != 0)
		GlyphSize = Point(FOriginal->Width / FNumGlyphs, FOriginal->Height);
	else
		GlyphSize = Point(0, 0);

	if (PopupArrow)	// Horz spacing of 3 each size, vertical of 2 each side
		PockSize = ArrowHorz ? Point(4 + PockHorzSpacing, 7 + 2 * PockVertSpacing):
								Point(7 + PockHorzSpacing, 4 + 2 * PockVertSpacing);
	else
		PockSize = Point(0, 0);

	bool LineLayout = (Layout == blGlyphLeft || Layout == blGlyphRight);
	if (!LineLayout && GlyphSize.x == 0)
	{
		Layout = blGlyphLeft;
		LineLayout = true;
	}

	// if there is no text or no bitmap, then Spacing is irrelevant
	if (Caption.Length() == 0 || GlyphSize.x == 0) Spacing = 0;

	if (Caption.Length() > 0)
	{
		*TextBounds = Rect(0, 0, Client->Right - Client->Left, 0);

		if (LineLayout)
			TextBounds->Right -= PockSize.x;

		if (WordWrap)
		{
			int Margin1 = 4;
			if (LineLayout && GlyphSize.x != 0)
			{
				int Spacing1 = Spacing == -1 ? 4: Spacing;
				TextBounds->Right -= GlyphSize.x + Spacing1;
				if (Margin != -1) Margin1 = Margin;
				else if (Spacing != -1) Margin1 = Spacing;
			}
			TextBounds->Right -= Margin1 * 2;
		}

		UINT uFormat = DT_CALCRECT + (WordWrap ? DT_WORDBREAK: 0);
		DrawText(Canvas->Handle, Caption.c_str(), Caption.Length(), (RECT*) TextBounds, uFormat);
		TextSize = Point(TextBounds->Right - TextBounds->Left, TextBounds->Bottom -
			TextBounds->Top);
	}
	else
	{
		*TextBounds = Rect(0, 0, 0, 0);
		TextSize = Point(0,0);
	}

	// If the layout has the glyph on the right or the left, then both the
	// text and the glyph are centered vertically.  If the glyph is on the top
	// or the bottom, then both the text and the glyph are centered horizontally.
	if (LineLayout)
	{
		GlyphPos->y = (ClientSize.y - GlyphSize.y + 1) / 2;
		TextPos.y = (ClientSize.y - TextSize.y + 1) / 2;
	}
	else
	{
		GlyphPos->x = (ClientSize.x - GlyphSize.x - PockSize.x + 1) / 2;
		TextPos.x = (ClientSize.x - TextSize.x + 1) / 2;
		if (GlyphSize.x == 0)
			PockPos->x = TextPos.x + TextSize.x;
		else
			PockPos->x = GlyphPos->x + GlyphSize.x;
	}

	// Adjust Margin and Spacing if user has specified -1
	if (Margin == -1)
	{
		if (Spacing == -1)
		{
			TotalSize = Point(GlyphSize.x + TextSize.x + PockSize.x,
				GlyphSize.y + TextSize.y);
			if (LineLayout)
				Margin = (ClientSize.x - TotalSize.x) / 3;
			else
				Margin = (ClientSize.y - TotalSize.y) / 3;
			Spacing = Margin;
		}
		else
		{
			TotalSize = Point(GlyphSize.x + Spacing + TextSize.x + PockSize.x,
				GlyphSize.y + Spacing + TextSize.y);
			if (LineLayout)
				Margin = (ClientSize.x - TotalSize.x + 1) / 2;
			else
				Margin = (ClientSize.y - TotalSize.y + 1) / 2;
		}
	}
	else
	{
		if (Spacing == -1)
		{
			TotalSize = Point(ClientSize.x - (Margin + GlyphSize.x + PockSize.x),
					ClientSize.y - (Margin + GlyphSize.y));
			if (LineLayout)
				Spacing = (TotalSize.x - TextSize.x) / 2;
			else
				Spacing = (TotalSize.y - TextSize.y) / 2;
		}
	}

	switch (Layout)
	{
	case blGlyphLeft:
		GlyphPos->x = Margin;
		TextPos.x = GlyphPos->x + GlyphSize.x + Spacing;
		PockPos->x = TextPos.x + TextSize.x;
		break;
	case blGlyphRight:
		PockPos->x = ClientSize.x - Margin - PockSize.x;
		GlyphPos->x = PockPos->x - GlyphSize.x;
		TextPos.x = GlyphPos->x - Spacing - TextSize.x;
		break;
	case blGlyphTop:
		GlyphPos->y = Margin;
		TextPos.y = GlyphPos->y + GlyphSize.y + Spacing;
		break;
	case blGlyphBottom:
		GlyphPos->y = ClientSize.y - Margin - GlyphSize.y;
		TextPos.y = GlyphPos->y - Spacing - TextSize.y;
		break;
	}
	if (GlyphSize.y == 0)
		PockPos->y = TextPos.y + (TextSize.y - PockSize.y) / 2;
	else
		PockPos->y = GlyphPos->y + (GlyphSize.y - PockSize.y) / 2;

	// fixup the result variables
	GlyphPos->x += Client->Left;
	PockPos->x += Client->Left;
	GlyphPos->y += Client->Top;
	PockPos->y += Client->Top;
	OffsetRect((RECT *)TextBounds, TextPos.x + Client->Left, TextPos.y + Client->Top);
}

TRect __fastcall TButtonGlyph::Draw(TCanvas* Canvas, const TRect* Client,
		const AnsiString& Caption, TButtonLayout Layout, int Margin, int Spacing,
		bool WordWrap, bool PopupArrow, bool ArrowHorz, bool Disabled, TGlyphType State)
{
	TPoint GlyphPos, PockPos;
	TRect Result;

	CalcButtonLayout(Canvas, Client, Caption, Layout, Margin, Spacing, WordWrap,
		PopupArrow, ArrowHorz, &GlyphPos, &PockPos, &Result);
	DrawButtonGlyph(Canvas, GlyphPos.x, GlyphPos.y, State);
	DrawButtonText(Canvas, Caption, &Result, State, WordWrap);
	if (PopupArrow) DrawPockMark(Canvas, PockPos.x, PockPos.y, ArrowHorz, Disabled);
	return Result;
}

//---------------------------------------------------------------------------

__fastcall TIEButton::TIEButton(TComponent* Owner)
	: TGraphicControl(Owner)
{
	SetBounds(0, 0, 25, 25);
	ControlStyle = TControlStyle() << csOpaque << csSetCaption << csCaptureMouse;
	DragMode = dmManual;
	FButtonPosition = bpForceCalc;		// Force calculation of state during Paint
	FGlyph = new TButtonGlyph();
	FGlyph->OnChange = GlyphChanged;
	FLayout = blGlyphLeft;
	FMargin = -1;
	FSleepingFontColor = clBlack;
	FSpacing = 4;
	ParentFont = true;
}
//---------------------------------------------------------------------------
__fastcall TIEButton::~TIEButton()
{
	delete FGlyph;
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::CalcStates()
{
	TButtonPosition BP;
	TGlyphType GT;

	// First, assume we are Explorer-style

	if (ComponentState.Contains(csDesigning))
	{
		BP = FDown && Enabled ? bpDown: bpUp;
		GT = Enabled ? (FDown ? gtExclusive: gtUp): gtDisabled;
	}
	else
	{
		if (!Enabled)
		{
			BP = bpFlat;
			GT = gtDisabled;
		}
		else
		{
			BP = FDown ? bpDown: (FMouseOverControl ? (FDragging ? bpDown: bpUp): bpFlat);
			GT = FDown ? gtExclusive: (FMouseOverControl ? (FDragging ? gtDown: gtUp): gtAsleep);
		}
	}

	if (FStandardButton)	// Cannot have the flat, asleep state
	{
		if (BP == bpFlat) BP = bpUp;
		if (GT == gtAsleep) GT = gtUp;
	}

	if (BP != FButtonPosition || GT != FButtonGlyph)
	{
		FButtonPosition = BP;
		FButtonGlyph = GT;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::Paint(void)
{
	if (FButtonPosition == bpForceCalc)	// Force calculation of state after creation?
	{
		CalcStates();
		return;
	}

	TRect PaintRect = DrawButtonSurface(Canvas, ClientRect, FButtonPosition,
			FTransparent, FSoftShading);

	if (FButtonGlyph == gtExclusive)
	{
		if (Pattern == 0) CreateBrushPattern();
		Canvas->Brush->Bitmap = Pattern;
		Canvas->FillRect(PaintRect);
	}

	Canvas->Font = Font;
	if (!ComponentState.Contains(csDesigning) && FButtonGlyph == gtAsleep)
		Canvas->Font->Color = SleepingFontColor;

	FGlyph->Draw(Canvas, &PaintRect, Caption, Layout, Margin, Spacing, WordWrap,
		DropDownArrow, DropDownArrowHorz, !Enabled, FButtonGlyph);
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::Loaded()
{
		TGraphicControl::Loaded();
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::MouseDown(TMouseButton Button,
	Classes::TShiftState Shift, int X, int Y)
{
	TGraphicControl::MouseDown(Button, Shift, X, Y);

	if (Enabled && Button == mbLeft) FDragging = true;

	CalcStates();

	if (FDragging && DropDownMenu && GroupIndex == 0)
	{
		POINT PopupPoint = {DropDownArrowHorz ? Width: 0, DropDownArrowHorz ? 0: Height};
		PopupPoint = ClientToScreen(PopupPoint);
		FDropDownMenu->Popup(PopupPoint.x, PopupPoint.y);
		FDragging = false;
		// We lose control of the mouse while menu is up; check to see if still over button
		GetCursorPos(&PopupPoint);
		PopupPoint = ScreenToClient(PopupPoint);
		FMouseOverControl = (PopupPoint.x >= 0 && PopupPoint.x < Width &&
			PopupPoint.y >= 0 && PopupPoint.y < Width);
		// Lose the first mouse down for this thread, just like IE
		TMsg Msg;
		PeekMessage(&Msg, 0, WM_LBUTTONDOWN, WM_LBUTTONDOWN, PM_REMOVE);
		CalcStates();
	}
}

//---------------------------------------------------------------------------
void __fastcall TIEButton::MouseMove(Classes::TShiftState Shift,
											int X, int Y)
{
	TGraphicControl::MouseMove(Shift, X, Y);

	TMessage Msg;
	Msg.WParam = 0;
	Msg.LParam = reinterpret_cast<long>(this);

	POINT MouseCoords = {X + Left, Y + Top};
	TControl* OverControl = Parent->ControlAtPos(MouseCoords, false);

	// A simple test that X and Y lie in the control's rectangle is not enough -
	// another button could be overlapping it.
	bool FMouseOverControl = (OverControl == this);

	// If the mouse button is down, the VCL captures the mouse, so other buttons
	// do not receive mouse messages. So to get them to pop up and down, we must
	// send the message ourselves.
	if (FDragging)
	{
		TIEButton* p = 0;
		if (OverControl != this) p = dynamic_cast<TIEButton*>(OverControl);

		if (FRaised && FRaised != p)
		{
			Msg.Msg = CM_MOUSELEAVE;
			FRaised->Dispatch(&Msg);
		}
		if (p)
		{
			Msg.Msg = CM_MOUSEENTER;
			p->Dispatch(&Msg);
		}
		FRaised = p;
	}

	// These few lines would not be necessary if CM_MOUSELEAVE worked when the
	// mouse exits the button in the case that it is passing to an obscuring app's window.
	// CM_MOUSELEAVE does not fire in this case (but does if ShowHint is on) - a
	// minor VCL bug. It does fire if we have mouse capture, so let's capture the mouse.
	if (FMouseOverControl)
	{
		if (GetCaptureControl() != this) SetCaptureControl(this);
	}
	else
	{
		// This section is only reached when FMouseOverControl = false. This
		// must mean the mouse has just left the control - we only got the event
		// because of the mouse capture in the above line. So release the mouse.
		if (!FDragging)
		{
			Msg.Msg = CM_MOUSELEAVE;
			Dispatch(&Msg);
			SetCaptureControl(0);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::MouseUp(TMouseButton Button,
	Classes::TShiftState Shift, int X, int Y)
{
	TGraphicControl::MouseUp(Button, Shift, X, Y);
	if (Button == mbLeft)
	{
		if (FDragging)
		{
			FDragging = false;
			if (FMouseOverControl && GroupIndex != 0) Down = !Down;
			CalcStates();
		}
		if (Enabled && DropDownMenu == 0) Click();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::Notification(TComponent* AComponent, TOperation Operation)
{
	if (Operation == opRemove && AComponent == FDropDownMenu)
		FDropDownMenu = 0;
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetDropDownArrow(bool Value)
{
	if (FDropDownArrow != Value)
	{
		FDropDownArrow = Value;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetDropDownArrowHorz(bool Value)
{
	if (FDropDownArrowHorz != Value)
	{
		FDropDownArrowHorz = Value;
		if (FDropDownArrow) Invalidate();
	}
}
//---------------------------------------------------------------------------
Graphics::TBitmap* __fastcall TIEButton::GetGlyph()
{
	return FGlyph->Glyph;
}
//---------------------------------------------------------------------------
TNGlyphs __fastcall TIEButton::GetNumGlyphs()
{
	return FGlyph->NumGlyphs;
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::GlyphChanged(TObject* Sender)
{
	Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::UpdateExclusive()
{
	TMessage Msg;

	if (FGroupIndex != 0 && Parent != 0)
	{
		Msg.Msg = CM_BUTTONPRESSED;
		Msg.WParam = FGroupIndex;
		Msg.LParam = reinterpret_cast<long>(this);
		Msg.Result = 0;
		Parent->Broadcast(&Msg);
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetAllowAllUp(bool Value)
{
	if (FAllowAllUp != Value)
	{
		FAllowAllUp = Value;
		UpdateExclusive();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetDown(bool Value)
{
	if (FGroupIndex == 0) Value = false;
	if (Value != FDown)
	{
		if (FDown && !FAllowAllUp) return;
		FDown = Value;
		CalcStates();
		if (Value) UpdateExclusive();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetGlyph(Graphics::TBitmap* Value)
{
	FGlyph->Glyph = Value;
	Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetGroupIndex(int Value)
{
	if (FGroupIndex != Value)
	{
		FGroupIndex = Value;
		UpdateExclusive();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetLayout(TButtonLayout Value)
{
	if (FLayout != Value)
	{
		FLayout = Value;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetMargin(int Value)
{
	if (Value != FMargin && Value >= -1)
	{
		FMargin = Value;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetNumGlyphs(TNGlyphs Value)
{
	if (Value <= 0) Value = 1;
	else if (Value > 4) Value = 4;
	if (Value != FGlyph->NumGlyphs)
	{
		FGlyph->NumGlyphs = Value;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetSleepingFontColor(TColor Value)
{
	if (Value != FSleepingFontColor)
	{
		FSleepingFontColor = Value;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetSoftShading(bool Value)
{
	if (Value != FSoftShading)
	{
		FSoftShading = Value;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetStandardButton(bool Value)
{
	if (Value != FStandardButton)
	{
		FStandardButton = Value;
		CalcStates();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetSpacing(int Value)
{
	if (Value != FSpacing)
	{
		FSpacing = Value;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetTransparent(bool Value)
{
	if (Value != FTransparent)
	{
		FTransparent = Value;
		if (FTransparent)
			ControlStyle >> csOpaque;
		else
			ControlStyle << csOpaque;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::SetWordWrap(bool Value)
{
	if (Value != FWordWrap)
	{
		FWordWrap = Value;
		Invalidate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::CMEnabledChanged(TMessage& Message)
{
	CalcStates();
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::CMButtonPressed(TMessage& Message)
{
	TIEButton* Sender;

	if (Message.WParam == FGroupIndex)
	{
		Sender = reinterpret_cast<TIEButton*>(Message.LParam);
		if (Sender != this)
		{
			if (Sender->Down && FDown)
			{
				FDown = false;
				CalcStates();
			}
			FAllowAllUp = Sender->AllowAllUp;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::CMDialogChar(TCMDialogChar& Message)
{
	if (IsAccel(Message.CharCode, Caption) && Enabled)
		Click();
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::CMFontChanged(TMessage& Message)
{
	Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::CMMouseEnter(TMessage& Message)
{
	FMouseOverControl = true;
	if (!Enabled || ComponentState.Contains(csDesigning)) return;
	CalcStates();
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::CMMouseLeave(TMessage& Message)
{
	FMouseOverControl = false;
	if (!Enabled || ComponentState.Contains(csDesigning)) return;
	CalcStates();
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::CMTextChanged(TMessage& Message)
{
	Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TIEButton::CMSysColorChange(TMessage& Message)
{
	FGlyph->Invalidate();
	FGlyph->CreateButtonGlyph(FButtonGlyph);
}
//---------------------------------------------------------------------------
namespace Explorerbutton
{
	void __fastcall Register()
	{
		TComponentClass classes[1] = {__classid(TIEButton)};
		RegisterComponents("Samples", classes, 0);
	}
}
//---------------------------------------------------------------------------
