//---------------------------------------------------------------------------
#ifndef IEButtonH
#define IEButtonH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
#include <vcl\Buttons.hpp>
//---------------------------------------------------------------------------

// Visual state of button surface on screen, and glyph type
enum TButtonPosition {bpUp, bpFlat, bpDown, bpForceCalc};
enum TGlyphType {gtUp = 0, gtDisabled, gtAsleep, gtExclusive, gtDown, gtLast = gtDown};
typedef int TNGlyphs;

// A TGlyphList object stores an array of TBitmaps, all of the same size.
// Each glyph stores where its various images are contained (by Index) in
// the GlyphList for that size.

class TGlyphList: public TImageList
{
	TBits* Used;
	int FCount;
	int __fastcall AllocateIndex();
	virtual __fastcall TGlyphList(TComponent* AOwner);
public:
	virtual __fastcall TGlyphList(TComponent* AOwner, int AWidth, int AHeight);
	virtual __fastcall ~TGlyphList();
	int __fastcall Add(Graphics::TBitmap* Image, Graphics::TBitmap* Mask);
	int __fastcall AddMasked(Graphics::TBitmap* Image, TColor MaskColor);
	void __fastcall Delete(int Index);
	__property int Count = {read = FCount};
};

// TGlyphCache stores an array of TGlyphLists; each containing Glyphs of
// unique dimensions. The idea is to have one unit-scope TGlyphCache instance.

class TGlyphCache
{
	TList* GlyphLists;
public:
	__fastcall TGlyphCache();
	virtual __fastcall ~TGlyphCache();
	TGlyphList* __fastcall GetList(int AWidth, int AHeight);
	void __fastcall ReturnList(TGlyphList* List);
	bool __fastcall Empty();
};

//---------------------------------------------------------------------------

class TButtonGlyph: public TObject
{
	friend class TIEButton;

	Graphics::TBitmap* FOriginal;
	TGlyphList* FGlyphList;
	int FIndexs[gtLast + 1];
	TColor FTransparentColor;
	TNGlyphs FNumGlyphs;

	TNotifyEvent FOnChange;

	void __fastcall GlyphChanged(TObject* Sender);
	void __fastcall SetGlyph(Graphics::TBitmap* Value);
	void __fastcall SetNumGlyphs(TNGlyphs Value);
	void __fastcall Invalidate();
	void __fastcall DrawPockMark(TCanvas* Canvas, int X, int Y, bool Vertical, bool Disabled);
	void __fastcall DrawButtonGlyph(TCanvas* Canvas, int X, int Y,
			TGlyphType State);
	void __fastcall DrawButtonText(TCanvas* Canvas, const AnsiString& Caption,
			TRect* TextBounds, TGlyphType State, bool WordWrap);
	void __fastcall CalcButtonLayout(TCanvas* Canvas, const TRect* Client,
		const AnsiString& Caption, TButtonLayout Layout, int Margin, int Spacing,
		bool WordWrap, bool PopupArrow, bool ArrowHorz, TPoint* GlyphPos, TPoint* PockPos, TRect* TextBounds);
public:
	__fastcall TButtonGlyph::TButtonGlyph();
	virtual __fastcall ~TButtonGlyph();
	// returns the text rectangle
	TRect __fastcall Draw(TCanvas* Canvas, const TRect* Client,
		const AnsiString& Caption, TButtonLayout Layout, int Margin, int Spacing,
		bool WordWrap, bool PopupArrow, bool ArrowHorz, bool Disabled, TGlyphType State);
	__property Graphics::TBitmap* Glyph = {read = FOriginal, write = SetGlyph};
	__property TNGlyphs NumGlyphs = {read = FNumGlyphs, write = SetNumGlyphs};
	__property TNotifyEvent OnChange = {read = FOnChange, write = FOnChange};
	int __fastcall CreateButtonGlyph(TGlyphType State);
};

class TIEButton : public TGraphicControl
{
private:
	bool FAllowAllUp;
	TButtonPosition FButtonPosition;
	TGlyphType FButtonGlyph;
	int FGroupIndex;					// Must appear before FDown
	bool FDown;							// For use with exclusivity
	bool FDragging;						// True iff mouse is down and not yet released
	bool FDropDownArrow;
	bool FDropDownArrowHorz;
	TPopupMenu* FDropDownMenu;
	TButtonGlyph* FGlyph;
	TButtonLayout FLayout;				// Text above/below/left/right of glyph?
	int FSpacing;						// Space between text and glyph
	int FMargin;						// Space between text/glyph and button edge
	bool FMouseOverControl;
	TIEButton* FRaised;					// Any other button forced up during mouse capture
	bool FSoftShading;					// Soft shading of the button edge
	bool FStandardButton;				// Set to true to get a normal, non-Explorer button
	TColor FSleepingFontColor;			// Colour of font when mouse is not over it
	bool FTransparent;					// Transparent button?
	bool FWordWrap;						// Wrap text in button?

	void __fastcall SetAllowAllUp(bool Value);
	void __fastcall SetDown(bool Value);
	void __fastcall SetDropDownArrow(bool Value);
	void __fastcall SetDropDownArrowHorz(bool Value);
	Graphics::TBitmap* __fastcall GetGlyph();
	void __fastcall SetGlyph(Graphics::TBitmap* Value);
	void __fastcall SetGroupIndex(int Value);
	void __fastcall SetMargin(int Value);
	TNGlyphs __fastcall GetNumGlyphs(void);
	void __fastcall SetNumGlyphs(TNGlyphs Value);
	void __fastcall SetSoftShading(bool Value);
	void __fastcall SetSpacing(int Value);
	void __fastcall SetStandardButton(bool Value);
	void __fastcall SetLayout(TButtonLayout Value);
	void __fastcall SetSleepingFontColor(TColor Value);
	void __fastcall SetTransparent(bool Value);
	void __fastcall SetWordWrap(bool Value);

	void __fastcall GlyphChanged(TObject* Sender);
	void __fastcall UpdateExclusive();
	void __fastcall CalcStates();

	MESSAGE void __fastcall CMButtonPressed(TMessage& Message);
	MESSAGE void __fastcall CMDialogChar(TCMDialogChar& Message);
	MESSAGE void __fastcall CMEnabledChanged(TMessage& Message);
	MESSAGE void __fastcall CMFontChanged(TMessage &Message);
	MESSAGE void __fastcall CMMouseEnter(TMessage &Message);
	MESSAGE void __fastcall CMMouseLeave(TMessage &Message);
	MESSAGE void __fastcall CMTextChanged(TMessage &Message);
	MESSAGE void __fastcall CMSysColorChange(TMessage &Message);

protected:
	virtual void __fastcall MouseMove(Classes::TShiftState Shift, int X, int Y);
	virtual void __fastcall Notification(TComponent* AComponent, TOperation Operation);
	virtual void __fastcall Paint(void);
	virtual void __fastcall Loaded(void);
	virtual void __fastcall MouseDown(TMouseButton Button,
		Classes::TShiftState Shift, int X, int Y);
	virtual void __fastcall MouseUp(TMouseButton Button,
		Classes::TShiftState Shift, int X, int Y);

public:
	__fastcall TIEButton(TComponent* Owner);
	virtual __fastcall ~TIEButton();

__published:
	__property int GroupIndex = {read = FGroupIndex, write = SetGroupIndex, default = 0};
	__property bool AllowAllUp = {read = FAllowAllUp, write = SetAllowAllUp, default = false};
	__property Caption;
	__property Cursor;
	__property bool Down = {read = FDown, write = SetDown};
	__property bool DropDownArrow = {read = FDropDownArrow, write = SetDropDownArrow, default = false};
	__property bool DropDownArrowHorz = {read = FDropDownArrowHorz, write = SetDropDownArrowHorz, default = false};
	__property Menus::TPopupMenu* DropDownMenu = {read = FDropDownMenu, write = FDropDownMenu, default = 0};
	__property Enabled;
	__property Font;
	__property Graphics::TBitmap* Glyph = {read = GetGlyph, write = SetGlyph};
	__property TButtonLayout Layout = {read = FLayout, write = SetLayout, default = blGlyphLeft};
	__property int Margin = {read = FMargin, write = SetMargin, default = -1};
	__property TNGlyphs NumGlyphs = {read = GetNumGlyphs, write = SetNumGlyphs, default = 1};
	__property ParentFont;
	__property ParentShowHint;
	__property PopupMenu;
	__property ShowHint;
	__property TColor SleepingFontColor = {read = FSleepingFontColor, write = SetSleepingFontColor, default = clBlack};
	__property bool SoftShading = {read = FSoftShading, write = SetSoftShading, default = false};
	__property int Spacing = {read = FSpacing, write = SetSpacing, default = 4};
	__property bool StandardButton = {read = FStandardButton, write = SetStandardButton, default = false};
	__property bool Transparent = {read = FTransparent, write = SetTransparent, default = false};
	__property Visible;
	__property bool WordWrap = {read = FWordWrap, write = SetWordWrap, default = false};

	__property OnClick;
	__property OnDragDrop;
	__property OnDragOver;
	__property OnEndDrag;
	__property OnMouseDown;
    __property OnMouseMove;
	__property OnMouseUp;
BEGIN_MESSAGE_MAP
	MESSAGE_HANDLER(CM_BUTTONPRESSED, TMessage, CMButtonPressed);
	MESSAGE_HANDLER(CM_ENABLEDCHANGED, TMessage, CMEnabledChanged);
	MESSAGE_HANDLER(CM_DIALOGCHAR, TCMDialogChar, CMDialogChar);
	MESSAGE_HANDLER(CM_FONTCHANGED, TMessage, CMFontChanged);
	MESSAGE_HANDLER(CM_MOUSEENTER, TMessage, CMMouseEnter);
	MESSAGE_HANDLER(CM_MOUSELEAVE, TMessage, CMMouseLeave);
	MESSAGE_HANDLER(CM_TEXTCHANGED, TMessage, CMTextChanged);
	MESSAGE_HANDLER(CM_SYSCOLORCHANGE, TMessage, CMSysColorChange);
END_MESSAGE_MAP(TGraphicControl);
};
//---------------------------------------------------------------------------
#endif
