/////////////////////////////////////////////////////////////////////////
// CGI Script to display records from a database
// Written by George Cross, Borland C++ Developer Support
// November 5 1996
//
// Build this source file using Borland C++ 5.0x as follows:
//
//   bcc32 -c thisfile.cpp
//   tlink32 c0x32 thisfile.obj,,,idapi32.lib import.lib cw32.lib,,
/////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <strstrea.h>
#include <idapi.h>
/////////////////////////////////////////////////////////////////////////
//  Prototypes and Globals
/////////////////////////////////////////////////////////////////////////

const char* gszAliasName = "DSHOURS";                //database name
const char* gszPutHourExe = "/cgi-dos/puthour.exe";  //exe to submit query to
const int MAX_CONTENT_LENGTH = 500;                  //used in GetCGIArgs

inline void __fastcall ErrorCheck(DBIResult result);
void DisplayBDEError(DBIResult result);
void PrepareToReadHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                                char** ppszRecBuf, CURProps* pcurProps,
                                char* pszEngineerName, char* pszProject);
void CloseHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                        char* pszRecBuf, FLDDesc* pfielddesc);
void PrintHeading(char* pszEngineerName, char* pszProject);
void PrintRecord(hDBICur* phCur, char* pRecBuf);
void PrintTable(hDBICur* phCur, char* pRecBuf,
                char* pszEngineerName, char* pszProject);
void QueryDatabase(hDBIDb* phDb, hDBICur* phQryCur,
                   char* pszEngineerName, char* pszProject);
void GetCGIArgs(char* EngineerName, char* Project);
void ConvertCharInString(char* pszString, char oldchar, char newchar);

/////////////////////////////////////////////////////////////////////////
//  Utility functions for calling IDAPI
/////////////////////////////////////////////////////////////////////////

//This inline function is used to throw exceptions if an IDAPI function
//fails.
inline void __fastcall ErrorCheck(DBIResult result)
{
  if (result != DBIERR_NONE) throw result;
}

//This function simply displays a message box notifying the user of an
//IDAPI function failure.
void DisplayBDEError(DBIResult result)
{
  char errstr[DBIMAXMSGLEN+1];
  DbiGetErrorString(result, errstr);
  cout << "IDAPI Error: " << errstr << endl;
}

#pragma option -w-aus -w-par
//This function just opens the 'HOURS' database and prepares to read it.
void PrepareToReadHoursDatabase(hDBIDb* phDb, hDBICur* phCur,
                                char** ppszRecBuf, CURProps* pcurProps,
                                char* pszEngineerName, char* pszProject)
{
  DBIResult result;
  DbiInit(NULL);
  result = DbiOpenDatabase((char*) gszAliasName,
                           NULL,
                           dbiREADWRITE,
                           dbiOPENSHARED,
                           NULL,
                           0,
                           NULL,
                           NULL,
                           phDb);
  ErrorCheck(result);

  QueryDatabase(phDb, phCur, pszEngineerName, pszProject);

  result = DbiGetCursorProps(*phCur, pcurProps);
  ErrorCheck(result);

  *ppszRecBuf = new char[pcurProps->iRecSize];
}

//This function executes an SQL query on the database
void QueryDatabase(hDBIDb* phDb, hDBICur* phQryCur,
                   char* pszEngineerName, char* pszProject)
{
  char buf[256];
  sprintf(buf,
          "%s%s%s%s%s%s",
          "SELECT DISTINCT Hours FROM hours.DB ",
          "WHERE (Project = '",
          pszProject,
          "') AND (Engineer = '",
          pszEngineerName,
          "')");

  DBIResult result =
    DbiQExecDirect(*phDb, qrylangSQL, buf, phQryCur);
  ErrorCheck(result);
}

#pragma option -w.aus -w.par
//This function just releases those resources allocated for using this
//database, and then closes the database
void CloseHoursDatabase(hDBIDb* phDb, hDBICur* phCur, char* pszRecBuf)
{
  if (*phCur)
    DbiCloseCursor(phCur);
  if (pszRecBuf)
    delete[] pszRecBuf;
  if (*phDb)
    DbiCloseDatabase(phDb);
  DbiExit();
}

/////////////////////////////////////////////////////////////////////////
// These are the functions which send HTML to STDOUT.
/////////////////////////////////////////////////////////////////////////

//This function prints the titles of each field
void PrintHeading(char* pszEngineerName, char* pszProject)
{
  //Put in '|' instead of ' ' for passing arguments to next script
  //on the command line.  This could be any value other than '|'
  ConvertCharInString(pszEngineerName, ' ', '|');
  ConvertCharInString(pszProject, ' ', '|');

  cout << "Content-type: text/html\n\n "
       << "<html>"
       << "<title>DS Extra Departmental Hours</title>" << endl
       << "<body><font size=+2>DS Hours Database<font size=+1><p>" << endl
       << "Your current hours for the shown project are <br>"
          "shown below.  Update them as you desire." << endl
       << "<form method=\"post\" action=\""
       << gszPutHourExe << "?"
       << pszEngineerName << "+"
       << pszProject
       << "\">" << endl;

  //Take '' back out
  ConvertCharInString(pszEngineerName, '|', ' ');
  ConvertCharInString(pszProject, '|', ' ');
}

//This function prints the trailing details on the form
void PrintFooting()
{
  cout << "<input type=\"submit\" value=\"Update Hours\">" ;
}

//This function prints an individual record
void PrintRecord(hDBICur* phCur, char* pRecBuf)
{
  double hours;

  DbiGetField(*phCur, 1, pRecBuf, (unsigned char*)&hours, NULL);
  cout << "<td><input type=text name=Hours value="
       << hours
       << "></td>";
}

//This function iterates through all records in the table.
void PrintTable(hDBICur* phCur, char* pRecBuf,
                char* pszEngineerName, char* pszProject)
{
  char buf[50];

  cout << "<table border cellspacing=5 cellpadding=6><tr>"
       << "<td>" << pszEngineerName << "</td>"
       << "<td>" << pszProject      << "</td>";
  DbiSetToBegin(*phCur);
  DbiGetNextRecord(*phCur, dbiNOLOCK, pRecBuf, NULL);
  PrintRecord(phCur, pRecBuf);

  cout << "</tr></table>";
}


/////////////////////////////////////////////////////////////////////////
//These functions get the arguments for us from STDIN
/////////////////////////////////////////////////////////////////////////

// This function is retrieves the engineer's name and the project
// from STDIN supplied by CGI
void GetCGIArgs(char* pszEngineerName, char* pszProject)
{
  char data[MAX_CONTENT_LENGTH];
  char* p1;
  char* p2;
  int length, pos;
  int i;

  //Determine the length of everything waiting in STDIN.
  //This information is stored in the CONTENT_LENGTH environment
  //variable and must be converted from its string value to an integer
  p1 = getenv("CONTENT_LENGTH");
  if (p1 != NULL)
    length = atoi(p1);
  else
    return;

  //Read in everything waiting in STDIN and terminate it with
  //the null character
  for (i=0;i<length;i++){
    fscanf(stdin, "%c", &data[i]);
  }
  data[i] = '\0';

  //Separate out the engineer's name and the project. Yes, this function
  //is specifically looking for arguments "Name" and "Project" in that order.
  p1 = strstr(data, "Name=");
  if (p1!=NULL){
    p1 = p1 + sizeof("Name=") - 1;
    p2 = strstr(p1, "&Project=");
    if (p2!= NULL){
      strncpy(pszEngineerName, p1, p2 - p1);
      pszEngineerName[p2 - p1] = '\0';
    }
    else
      return;
  }
  else
    return;
  strcpy(pszProject, p2 + sizeof("&Project=") - 1);

  //Convert any '+' symbols to spaces.  CGI forces this on us.
  ConvertCharInString(pszEngineerName, '+', ' ');
  ConvertCharInString(pszProject, '+', ' ');

  return;
}

//This function converts any '+' symbols to spaces
void ConvertCharInString(char* pszString, char oldchar, char newchar)
{
  char* p = pszString;
  while (p){
    p = strchr(p, oldchar);
    if (p)
      *p = newchar;
  }
}


/////////////////////////////////////////////////////////////////////////
// This function is the main entry point for this module
/////////////////////////////////////////////////////////////////////////
int main()
{
  hDBIDb hDb = NULL;               //Handle to the database
  hDBICur hCur = NULL;             //Handle to the 'HOURS' table
  char* pszRecBuf = NULL;          //Buffer to hold an entire record
  CURProps curProps;               //Structure to hold table properties
  char szEngineerName[15];         //Buffer to hold the Engineer's Name
  char szProject[50];              //Buffer to hold the Project Name

  try{
    GetCGIArgs(szEngineerName, szProject);
    PrepareToReadHoursDatabase(&hDb,&hCur,&pszRecBuf,&curProps,
                               szEngineerName,szProject);
    PrintHeading(szEngineerName, szProject);
    PrintTable(&hCur,pszRecBuf,szEngineerName,szProject);
    PrintFooting();
    CloseHoursDatabase(&hDb, &hCur, pszRecBuf);
  }
  catch(DBIResult result){
    DisplayBDEError(result);
    CloseHoursDatabase(&hDb, &hCur, pszRecBuf);
  }

  return 0;
}


