#include <windows.h>
#include <windowsx.h>
#include "splitwnd.h"
#include "resource.rh"

#define DIVIDER_WIDTH     7
static RECT g_rcClient;       // Save the client area so the controls
                              //   can remain proportionally sized
static RECT g_rcDivider;      // Position of the divider between
                              //   the controls
static LONG g_lPrevPos;       // Previous mouse position for resizing
static double g_dDividerProportion;   // Proportion of the screen that the
                                      //   is at divider
static HCURSOR g_hcurResize;          // Resizing cursor handle


void Dlg_OnPaint(HWND hwnd)
{
   PAINTSTRUCT ps;
   HDC      hdc;

   hdc = BeginPaint(hwnd, &ps);

   DrawEdge(hdc, &g_rcDivider,
            EDGE_RAISED, BF_TOP | BF_BOTTOM | BF_MIDDLE);

   EndPaint(hwnd, &ps);
}


void Dlg_OnSize (HWND hwnd, UINT state, int cx, int cy)
{
    // store the client area size for the next time we have to do this
    g_rcClient.right = cx;
    g_rcClient.bottom = cy - g_rcClient.top;

    // set the new divider position to a proportional position
	 g_rcDivider.top = (LONG) ((g_rcClient.bottom + g_rcClient.top)
                              * g_dDividerProportion);
    g_rcDivider.bottom = g_rcDivider.top + DIVIDER_WIDTH;
    g_rcDivider.right = cx;

    // Update the positions of the controls
    SetWindowPos(GetDlgItem(hwnd, IDC_LISTBOX1),
               NULL,
               0,
               g_rcClient.top,
               cx,
               g_rcDivider.top - g_rcClient.top,
               SWP_NOZORDER);

    SetWindowPos(GetDlgItem(hwnd, IDC_EDIT1),
               NULL,
               0,
               g_rcDivider.bottom + g_rcClient.top,
               cx,
               g_rcClient.bottom - g_rcDivider.bottom + g_rcClient.top,
               SWP_NOZORDER);
}


void Dlg_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags)
{
  if (GetCapture() != NULL)
  {
    DrawResizeLine(hwnd, g_lPrevPos);   // remove the previous line
    DrawResizeLine(hwnd, y);            // draw the new resizing line in
                                        // .. the current position
    g_lPrevPos = y;                     // save this position for next
  }                                     // .. time
  else
  {
    // if the mouse is over the resizing line, change the cursor
    POINT pt;

    pt.x = x; pt.y = y;
    if (PtInRect(&g_rcDivider, pt))
       SetCursor(g_hcurResize);
  }
}


void Dlg_OnLButtonDown(HWND hwnd, bool fDoubleClick, int x, int y, UINT keyFlags)
{
  POINT pt;

  pt.x = x;
  pt.y = y;

  // Check to see if the mouse is in the divider
  if (PtInRect(&g_rcDivider, pt))
  {
    SetCursor(g_hcurResize);
    SetCapture(hwnd);             // keep track of the mouse until the
                                  // ..button is released;
    DrawResizeLine(hwnd, pt.y);   // draw the resizing rectangle
    g_lPrevPos = pt.y;            // save the position so the resize
                                  // ..rectangle can be moved
  }
}


void Dlg_OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags)
{
  if (GetCapture() != NULL)
  {
    SetCursor(g_hcurResize);
    // clean up from resize operation, release the capture and
    // remove the resizing line from the window
    ReleaseCapture();
    DrawResizeLine(hwnd, g_lPrevPos);

    // update the divider position
    g_rcDivider.top = y;
    g_rcDivider.bottom = g_rcDivider.top + DIVIDER_WIDTH;

    // Update the positions of the controls
    SetWindowPos(GetDlgItem(hwnd, IDC_LISTBOX1),
               NULL,
               0,
               g_rcClient.top,
               g_rcClient.right,
               g_rcDivider.top - g_rcClient.top,
               SWP_NOZORDER);

    SetWindowPos(GetDlgItem(hwnd, IDC_EDIT1),
               NULL,
               0,
               g_rcDivider.bottom + g_rcClient.top,
               g_rcClient.right,
               g_rcClient.bottom - g_rcDivider.bottom + g_rcClient.top,
               SWP_NOZORDER);


    // find out where the divider was in proportion to the window size
    g_dDividerProportion = (double)(g_rcDivider.top) /
          (double)(g_rcClient.bottom + g_rcClient.top);

    // update the window
    InvalidateRect(hwnd, NULL, TRUE);
    UpdateWindow(hwnd);
  }
}


bool Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
  HWND hwndLB,hwndEB;
  RECT rcClient;
  GetClientRect(hwnd, &rcClient);

  hwndEB = CreateWindow("edit",
                        "",
                        WS_CHILD|WS_VISIBLE,
                        rcClient.left,
                        rcClient.top,
                        rcClient.right,
                        rcClient.bottom,
                        hwnd,
                        (HMENU)IDC_EDIT1,
                        GetModuleHandle(NULL),
                        NULL);

  hwndLB = CreateWindow("listbox",
                        "",
                        WS_CHILD|WS_VISIBLE,
                        rcClient.left,
                        rcClient.top,
                        rcClient.right,
                        rcClient.bottom,
                        hwnd,
                        (HMENU)IDC_LISTBOX1,
                        GetModuleHandle(NULL),
                        NULL);

  // Set the initial size of the client area and divider position
  GetClientRect(hwnd, &g_rcClient);

  // The initial divider proportion is 50%
  g_dDividerProportion = 0.50;

  // The initial divider position is mid way down the client area
  g_rcDivider.top = (LONG) ((g_rcClient.bottom - g_rcClient.top)
                         * g_dDividerProportion) + g_rcClient.top;
  g_rcDivider.bottom = g_rcDivider.top + DIVIDER_WIDTH;
  g_rcDivider.left = 0;
  g_rcDivider.right = g_rcClient.right;

  // Call the size routine since Win95 and NT don't initally send
  // WM_SIZE to the window
  Dlg_OnSize(hwnd, 0, g_rcClient.right, g_rcClient.bottom);

  // Load the resizing cursor from the resources, if a problem occurs,
  // default to the system defined arrow cursor
  g_hcurResize = LoadCursor(GetModuleHandle(NULL),
                            MAKEINTRESOURCE(IDC_RESIZE));
  if (!g_hcurResize)
     g_hcurResize = LoadCursor(NULL, MAKEINTRESOURCE(IDC_ARROW));

  return true;
}


void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
  switch (id){
    case IDCANCEL:
      EndDialog(hwnd,id);
      break;
  }
}


bool CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg,
                        WPARAM wParam, LPARAM lParam)
{
  bool fProcessed = true;
  switch (uMsg){
    HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
    HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);
    HANDLE_MSG(hDlg, WM_SIZE, Dlg_OnSize);
    HANDLE_MSG(hDlg, WM_PAINT, Dlg_OnPaint);
    HANDLE_MSG(hDlg, WM_LBUTTONDOWN, Dlg_OnLButtonDown);
    HANDLE_MSG(hDlg, WM_LBUTTONUP, Dlg_OnLButtonUp);
    HANDLE_MSG(hDlg, WM_MOUSEMOVE, Dlg_OnMouseMove);

    default:
      fProcessed = false;
      break;
  }
  return fProcessed;
}


int APIENTRY WinMain (HINSTANCE hinstExe, HINSTANCE hinstPrev,
                      LPSTR lpszCmdLine, int nCmdShow)
{
  DialogBox(hinstExe, MAKEINTRESOURCE(DIALOG_1),
            NULL, (DLGPROC)Dlg_Proc);
  return 0;
}
