///////////////////////////////////////////////////////

#include "Matrix.h"
#include "except.h"
///////////////////////////////////////////////////////////
/////////////////// members of orthodox canonical form
Matrix::Matrix()
 : n(1), r(1), c(1)
{ //default constructor
  try{
  z = new double[1];
  }
  catch( xalloc &x ){
    z = NULL;
    errormsg("allocation failure in default contructor");
  }
}

Matrix::Matrix( Matrix &a )
 : n(a.n), r(a.r), c(a.c)
{ //copy constructor
  try{
  z = new double[a.n];
  }
  catch( xalloc &x ){
    z = NULL;
    errormsg("allocation failure in copy contructor");
  }
  for( int i=1; i<=n; i++) (*this)(i) = a(i);
}

Matrix &Matrix::operator=(const Matrix &a )
{ //assignment operator
  if (this == &a) return *this;
  if( n != a.n ){
    delete[] z;
    try{
      z = new double[a.n];
    }
    catch( xalloc &x ){
      z = NULL;
      errormsg("allocation failure in assignment operator");
    }
    n = a.n; r = a.r; c = a.c;
  }
  for( int i=1; i<=n; i++) (*this)(i) = a(i);
  return *this;
}

Matrix::~Matrix()
{  //destructor
   delete[] z;
}

//////////////////////// more constructors

Matrix::Matrix(int rr, int cc, double v )
  : r(rr), c(cc), n(rr*cc)
{ // rxc constructor - with initialization
  if( n<=0 )
    errormsg("n<=0 in initialized constructor");
    try{
      z = new double[n];
    }
    catch( xalloc &x ){
      z = NULL;
      errormsg("allocation failure in constructor");
    }
  for( int i=1; i<=n; i++) (*this)(i) = v;
}

Matrix::Matrix(int rr, int cc, double *carr)
 : r(rr), c(cc), n(rr*cc)
{ // rxc constructor - copy vector into it.
  if( n<=0 )
    errormsg("n<=0 in vector constructor");
  try{
    z = new double[n];
  }
  catch( xalloc &x ){
    z = NULL;
    errormsg("allocation failure in constructor");
  }
  for( int i=1; i<=n; i++) (*this)(i) = carr[i-1];
}

///////////////////// indexing

double & Matrix::operator()( int i ) const
{ // index matrix as vector.
  if( i < 1 || i > n || !z)
    errormsg("Index out of range or null matrix");
  return z[i-1];

}

double & Matrix::operator()( int i, int j) const
{ // matrix indexing
  if( i < 1 || i > r || j < 1 || j > c || !z)
    errormsg("Index out of range or null matrix");
  return z[(i-1)*c+(j-1)];
}

////////////////////// operators
////////////// addition
Matrix Matrix::operator+(const const Matrix &a )
{
  if( a.r != r || a.c != c)
    errormsg("Matrices do not conform in addition");
  Matrix t(r,c);
  for( int i=1; i<=n; i++) t(i) = (*this)(i) + a(i);
  return t;
}
Matrix Matrix::operator+( double b )
{
  Matrix t(r,c);
  for( int i=1; i<=n; i++) t(i) = (*this)(i) + b;
  return t;
}
Matrix operator+(double b, const Matrix &a )
{
  Matrix t(a.R(),a.C());
  for( int i=1; i<=a.N(); i++) t(i) = a(i) + b;
  return t;
}
Matrix& Matrix::operator+=(const Matrix &a)
{
  if( a.r != r || a.c != c)
    errormsg("Matrices do not conform in addition");
  for(int i=1; i<=n; i++) (*this)(i) += a(i);
  return *this;
}
Matrix& Matrix::operator+=(double b)
{
   for(int i=1; i<=n; i++) (*this)(i) += b;
   return *this;
}
///////////////////////////// subtraction
Matrix Matrix::operator-( const Matrix &a )
{
  if( a.r != r || a.c != c)
    errormsg("Matrices do not conform in subraction");
  Matrix t(r,c);
  for( int i=1; i<=n; i++) t(i) = (*this)(i) - a(i);
  return t;
}
Matrix Matrix::operator-( double b )
{
  Matrix t(r,c);
  for( int i=1; i<=n; i++) t(i) = (*this)(i) - b;
  return t;
}
Matrix operator-(double b, const Matrix &a )
{
  Matrix t(a.R(),a.C());
  for( int i=1; i<=a.N(); i++) t(i) = b-a(i);
  return t;
}
Matrix& Matrix::operator-=(const Matrix &a)
{
  if( a.r != r || a.c != c)
    errormsg("Matrices do not conform in subtraction");
  for(int i=1; i<=n; i++) (*this)(i) -= a(i);
  return *this;
}
Matrix& Matrix::operator-=(double b)
{
   for(int i=1; i<=n; i++) (*this)(i) -= b;
   return *this;
}


///////////// negation
Matrix Matrix::operator-()
{
  Matrix t(r,c);
  for( int i=1; i<=n; i++) t(i) = -(*this)(i);
  return t;
}

//////////////// elementwise multiplication
Matrix Matrix::operator%( const Matrix &a )
{
  if( a.r != r || a.c != c)
    errormsg("Matrices do not conform in elementwise mult");
  Matrix t(r,c);
  for( int i=1; i<=n; i++) t(i) = (*this)(i) * a(i);
  return t;
}

Matrix Matrix::operator*( const Matrix &a )
{
  if( c != a.r )
    errormsg("Matrices do not conform in multiplication");
  Matrix t(r,a.c, 0.0);
  long double sum;
  for( int i=1; i<=r; i++)
    for( int j=1; j<=a.c; j++){
       sum = 0.0;
       for( int k=1; k<=c; k++)
          sum += ((long double)(*this)(i,k))*((long double)a(k,j));
       t(i,j) = (double)sum;
    }
  return t;
}
Matrix Matrix::operator*( double b )
{
  Matrix t(r,c);
  for( int i=1; i<=n; i++) t(i) = (*this)(i)*b;
  return t;
}
Matrix operator*(double b, const Matrix &a )
{
  Matrix t(a.R(),a.C());
  for( int i=1; i<=a.N(); i++) t(i) = b*a(i);
  return t;
}
Matrix& Matrix::operator*=(const Matrix &a)
{
  if( c != a.r )
    errormsg("Matrices do not conform in multiplication");
  Matrix t=(*this)*a;
  *this = t;
  return *this;
}
Matrix& Matrix::operator*=(double b)
{
  for( int i=1; i<=n; i++) (*this)(i)*=b;
  return *this;
}

Matrix Matrix::operator/(const Matrix &a)
{
  if( a.r != r || a.c != c)
    errormsg("Matrices do not conform in division");
  Matrix t(r,c);
  for( int i=1; i<=n; i++)
    if( a(i) == 0.0 )
      errormsg("zero divisor in elementwise division");
    else
      t(i) = (*this)(i)/a(i);
  return t;
}
Matrix Matrix::operator/( double b )
{
  if( b==0.0 )
    errormsg("zero divisor in scalar division");
  Matrix t(r,c);
  for( int i=1; i<=n; i++) t(i) = (*this)(i)/b;
  return t;
}
Matrix& Matrix::operator/=(const Matrix &a)
{
  if( a.r != r || a.c != c)
    errormsg("Matrices do not conform in division");
  for( int i=1; i<=n; i++)
    if( a(i) == 0.0 )
      errormsg("zero divisor in elementwise division");
    else
     (*this)(i) /=a(i);
  return *this;
}
Matrix& Matrix::operator/=(double b)
{
  if( b==0.0 )
    errormsg("zero divisor in scalar division");
  for( int i=1; i<=n; i++) (*this)(i)/=b;
  return *this;
}

/////////////////// matrix functions

Matrix Matrix::t()
{ // transpose
  Matrix t(c,r);
  for( int i=1; i<=r; i++)
    for(int j=1; j<=c; j++)
      t(j,i) = (*this)(i,j);
  return t;
}

//// sweep out rows k1 through k2
// operates in place
Matrix Matrix::Sweep(int k1, int k2)
{

  double eps=1.0e-8;
  double d;
  int i, j, k, it;

  if (c != r)
     Matrix::errormsg("sweep: matrix a not square");

  if (k2 < k1) { k = k1; k1 = k2; k2 = k; }

  for (k = k1; k <= k2; k++) {
    if ( fabs( (*this)(k, k) ) < eps)
      for (it = 1; it <= r; it++)
        (*this)(it, k) = (*this)(k, it) = 0.0;
    else {
      d = 1.0 / (*this)(k, k);
      (*this)(k, k) = d;
      for (i = 1; i <= r; i++)
        if (i != k) (*this)(i, k) *=  -d;
      for (j = 1; j <= r; j++)
        if (j != k) (*this)(k, j) *= d;
        for (i = 1; i <= r; i++) {
          if (i != k) {
            for (j = 1; j <= r; j++) {
              if (j != k)
                 (*this)(i, j) +=
                    (*this)(i, k) * (*this)(k, j) / d;
            } // end for j
          } // end if i != k
        } // end for i
    } // end else
  } // end for k
  return *this;
}   //sweep

///// inversion
Matrix Matrix::Inv()
{
   if (c != r)
     errormsg("INVERSE: matrix a not square");
   Matrix b= *this;
   b.Sweep(1, r);
   return b;
}

ostream &operator<<(ostream &s, const Matrix &m)
{
   for( int i=1; i<=m.R(); i++){
     for( int j=1; j<=m.C(); j++)
       s << m(i,j) << " ";
     s << endl;
   }
   s << endl;
   return s;
}

/////////////////////////////////////////////////

Matrix I(int n)
{ // nxn identity matrix
  Matrix t(n,n,0.0);
  for( int i=1; i<=n; i++) t(i,i) = 1.0;
  return t;
}
Matrix Inv(Matrix &a)
{
	return a.Inv();
}
Matrix Sweep(Matrix &a, int l, int t)
{
   return a.Sweep(l,t);
}
Matrix Tran(Matrix &a)
{
   return a.t();
}
Matrix Submat(Matrix &a, int lr, int lc, int r, int c)
{
   if ((r > a.R()) || (c > a.C()))
     a.errormsg("SUBMAT: index out of range");

   int r2 = r - lr + 1;
   int c2 = c - lc + 1;
   Matrix b( r2, c2);
   int lrm1 = lr - 1;
   int lcm1 = lc - 1;
   for (int i = 1; i <= r2; i++) {
       for (int j = 1; j <= c2; j++) b(i, j) = a(lrm1 + i, lcm1 + j);
   }
   return b;
}   /* submat */
Matrix Row(int i, Matrix &a)
{
  Matrix t = Submat(a,i,1,i,a.C());
  return t;
}
Matrix Col(int i, Matrix &a)
{
  Matrix t = Submat(a,1,i,a.R(),i);
  return t;
}

Matrix Ch(Matrix &a, Matrix &b)
{
   if (a.R() != b.R())
      a.errormsg("CH: matrices have different number of rows");

   Matrix c(a.R(), a.C()+b.C());
   int i;
   for (i = 1; i <= a.R(); i++) {
       for (int j = 1; j <= a.C(); j++)
          c(i, j) = a(i, j);
   }
   int ii;
   for (i = 1, ii = 1; i <= b.R(); i++, ii++) {
      for (int j = 1, jj = 1; j <= b.C(); j++, jj++)
           c(ii, jj + a.C()) = b(i, j);
   }
   return c;
} /* ch */

///// vertical concatination
Matrix Cv(Matrix &a, Matrix &b)
{
  if(a.C() != b.C())
     a.errormsg("CV: matrices have different number of columns");

  Matrix c( a.R()+b.R(), a.C());
  int i;
  for ( i = 1; i <= a.R(); i++) {
      for (int j = 1; j <= a.C(); j++)
          c(i, j) = a(i, j);
  }
  int ii;
  for (i = 1, ii = 1; i <= b.R(); i++, ii++) {
      for (int j = 1, jj = 1; j <= b.C(); j++, jj++)
          c(ii + a.R(), jj) = b(i, j);
  }
  return c;
} /* cv */

