
#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>
#include <math.h>

#ifndef _MATRIX_
#define _MATRIX_

// a class for matrix exceptions thrown in
// the errormsg function
class MatrixException
{
   public:
     MatrixException(){ msg=NULL; }
     char *msg;
     MatrixException(char *m){ msg=m; }
};

// a simple double precision matrix class
class Matrix
{
public:
  // orthodox canonical form
  Matrix();
  Matrix(Matrix &a);
  Matrix &operator=(const Matrix &a);
  virtual ~Matrix();

  // constructor - default value of 0
  Matrix(int rr, int cc, double v=0.0 );
  // copy an array into a matrix
  Matrix(int rr, int cc, double *double_array);

  // indexing from 1 to n, and r,c
  double & operator()( int i ) const;
  double & operator()( int i, int j) const;

  // error exit
  static void errormsg(char *errormsg){
     throw MatrixException(errormsg);
  }
  // read access to private members
  int R() const { return r; }
  int C() const { return c; }
  int N() const { return n; }

  // operators
  Matrix operator+(const Matrix &a); //addition
  Matrix operator+(double b);
  friend Matrix operator+(double b,const Matrix &a);
  Matrix& operator+=(const Matrix &a);
  Matrix& operator+=(double b);

  Matrix operator-(const Matrix &a); //subtraction
  Matrix operator-(double b);
  friend Matrix operator-(double b, const Matrix &a);
  Matrix& operator-=(const Matrix &a);
  Matrix& operator-=(double b);

  Matrix operator-();          //negation
  Matrix operator%(const Matrix &a); //elementwise multiplication

  Matrix operator*(const Matrix &a); //matrix multiplication
  Matrix operator*(double b);
  friend Matrix operator*(double b, const Matrix &a);
  Matrix& operator*=(const Matrix &a);
  Matrix& operator*=(double b);

  Matrix operator/(const Matrix &a); //elementwise division
  Matrix operator/(double a);
  Matrix& operator/=(const Matrix &a);
  Matrix& operator/=(double b);

  Matrix Sweep(int k1, int k2); // g2 sweep
  Matrix Inv();                 // g2 sweep inversion
  Matrix t();                   // transpose


private:
  int r,c,n;
  double *z;

};

ostream &operator<<(ostream &s,const Matrix &m);
Matrix I(int n);
Matrix Sweep(Matrix &a, int l, int t);
Matrix Inv(Matrix &a);
Matrix Tran(Matrix &a);
Matrix Submat(Matrix &a, int lr, int lc, int r, int c);
Matrix Ch(Matrix &a, Matrix &b);
Matrix Cv(Matrix &a, Matrix &b);

#endif


