////////////////////////////////////////////////////////////////////////////////
// TSplitterControl class - Oxsoft Heart V5.0
//
// Class that allows to resize two panels (or so) which are separated by a kind
// of splitter
//
// by Alan GARNY
//    gry@physiol.ox.ac.uk
//    http://www.physiol.ox.ac.uk/~gry
//
//  Copyright 1996-97
////////////////////////////////////////////////////////////////////////////////

#ifndef SplitterControlH
#define SplitterControlH

////////////////////////////////////////////////////////////////////////////////
// Constants

extern const int _NO_SIZE_;   // To use for "MinSize" and "MaxSize" if we don't
                              // want a minimum/maximum size for "TargetControl"

////////////////////////////////////////////////////////////////////////////////

class TSplitterControl
{
   private:
      TForm *Form;   // The form from which we create the split control
      TControl *SplitterControl;   // The split bar
      TControl *TargetControl;   // The left (or so) control to resize
      bool ResizeVertically;   // To know if we resize horizontally or
                               // vertically
      int MinSize, MaxSize;   // Minimum and maximum size that "TargetControl"
                              // can have
      TPoint SplitterPos;   // Current position of the splitter (useful when
                            // resizing)
      int OrigPos;   // Abscisse or ordinate (depends on "ResizeVertically" of
                     // the mouse when the resizing starts
      bool Working;   // To know if we are resizing or not

      void __fastcall DrawSplitterControl();
   public:
      __fastcall TSplitterControl(TForm *aForm);

      void __fastcall OnMouseDown(const int &aX, const int &aY,
                                  TControl *aSplitterControl,
                                  TControl *aTargetControl,
                                  TControl *aOtherTargetControl,
                                  const int &aMinSize,
                                  const int &aMaxSize,
                                  const bool &aResizeVertically);
      // Note:
      // - aSplitterControl: name of the panel (or so) which is used as a
      //   splitter and which the "Align" property is set to "alLeft" (or so),
      // - aTargetControl: panel (or so) which the "Align" property is set to
      //   "alLeft" (or so), the other panel (or so) must have its "Align"
      //   property set to "alClient",
      // - aOtherTargetControl: analogous to "aTargetControl", except that in
      //   this case "Align" is to be set to "alClient"
      // - aMinSize: minimum width/height that "aTargetControl" can have,
      // - aMaxSize: maximum width/height that "aTargetControl" can have.
      // - aResizeVertically: specifies wether we want to resize vertically or
      //   horizontally
      void __fastcall OnMouseMove(const int &aX, const int &aY);
      void __fastcall OnMouseUp(const int &aX, const int &aY);
};

////////////////////////////////////////////////////////////////////////////////

#endif

// End of file
////////////////////////////////////////////////////////////////////////////////

