//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Swanny1a.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TForm1 *Form1;

#define MaxSymbols 6

// The symbol structure describes where each symbol's top left corner is (x,y)
// and how fast it is moving left-right (dx) and up-down (dy).
// If dx < 0 the Symbol moves left, if dx > 0 the object moves right.
// If dy < 0 the Symbol moves up, if dy > 0 the object moves down.

struct SymbolType
{ int x;
  int y;
  int dx;
  int dy;
} Symbol[MaxSymbols];

// WallRect is the rectangle that covers the whole wall
// in WallImage, DrawBox & ShowBox.
// ImageRect is the rectangle that covers the symbol's Image in SymbolImage.
// MaskRect is the rectangle that covers the symbol's Mask in SymbolImage.
// DestRect is the calculated rectangle that covers the Symbols position
// in DrawBox.

#define WallRect  Rect( 0, 0, 416, 319)
#define ImageRect Rect( 0, 0, 63, 63)
#define MaskRect  Rect( 64, 0, 127, 63)
#define DestRect( x, y) Rect( x , y , x + 63, y + 63)

// MaxX is how far to the right the symbols left side can go
// MaxY is how far down the symbols top side can go.
// We need these because the symbols position is based on it's top,left corner

#define MaxX 352
#define MaxY 256

// SpeedX is the number of pixels an object moves left-right per timer event
// SpeedY is the number of pixels an object moves up-down per timer event

#define SpeedX 2
#define SpeedY 2

//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner)
{
};
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
  int t;
  for (t=0; t<MaxSymbols; t++)
  {
    // Prepare the Symbols
    // Calculate their intial x,y location
  	Symbol[t].x = random(MaxX);
  	Symbol[t].y = random(MaxY);
    // Calculate their intial dx,dy movement
    if ( random(2))            	// Flip a coin.
      Symbol[t].dx = SpeedX;    // If heads move right
    else                        // or
      Symbol[t].dx = - SpeedX;  // if tails move left.
    if ( random(2))             // Flip another coin.
      Symbol[t].dy = SpeedY;    // If heads move down
    else                        // or
      Symbol[t].dy = - SpeedY;  // if tails move up
  }
};
//---------------------------------------------------------------------------
void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
  // First we draw everything to DrawBox.
  // Then we copy DrawBox to ShowBox.
  // We will use the variable 't' to step through the symbols
  int t;
  // Draw the wall
  DrawBox->Canvas->CopyMode = cmSrcCopy;
  DrawBox->Canvas->CopyRect( WallRect,
  		             		 WallImage->Canvas,
                     		 WallRect );
  // Draw & move each of the symbols
  for (t=0; t<MaxSymbols; t++)
  {
    // Draw this symbol's mask first
    DrawBox->Canvas->CopyMode = cmSrcAnd;
    DrawBox->Canvas->CopyRect( DestRect( Symbol[t].x, Symbol[t].y ),
    				           SymbolImage->Canvas,
                               MaskRect );
    // Then draw this symbol's image
    DrawBox->Canvas->CopyMode = cmSrcPaint;
    DrawBox->Canvas->CopyRect( DestRect( Symbol[t].x, Symbol[t].y ),
    						   SymbolImage->Canvas,
                          	   ImageRect );
    // Now move this symbol
    Symbol[t].x += Symbol[t].dx;
    Symbol[t].y += Symbol[t].dy;
    // Now check for bouncing of the left & right walls
    if (Symbol[t].dx == SpeedX)
    {
      if ( Symbol[t].x >= MaxX )
      {
        // Bouncing of the right wall
        Symbol[t].x = MaxX - (Symbol[t].x - MaxX) - 1;
        Symbol[t].dx = 0 - SpeedX;
      }
    }
    else
    {
      if ( Symbol[t].x < 0 )
      {
        // Bouncing of the left wall
        Symbol[t].x = 0 - Symbol[t].x;
        Symbol[t].dx = SpeedX;
      }
	}
    // Now check for bouncing of the top & bottom walls
    if (Symbol[t].dy == SpeedY)
    {
      if ( Symbol[t].y >= MaxY )
      {
        // Bouncing of the bottom wall
        Symbol[t].y = MaxY - (Symbol[t].y - MaxY) - 1;
        Symbol[t].dy = 0 - SpeedY;
      }
    }
    else
    {
      if ( Symbol[t].y < 0 )
      {
        // Bouncing of the top wall
        Symbol[t].y = 0 - Symbol[t].y;
        Symbol[t].dy = SpeedY;
      }
	}
  }
  // Now copy the buffer to visible area
  ViewBox->Canvas->CopyMode = cmSrcCopy;
  ViewBox->Canvas->CopyRect( WallRect,
                             DrawBox->Canvas,
                             WallRect );
};
//---------------------------------------------------------------------------