#include "matrix.h"
#include "stdlib.h"

double z()
{  // a cheap standard normal rng.
   // u(0,1) have mean 0.5, and variance 1/12,
   // so the sum of 12 u(0,1) has mean 6,
   // a variance of 1 = 12/12, and an approximate
   // normal distribution.
   double rannorm=0;
   for(int i=1; i<=12; i++)
     rannorm += ((double)rand())/((double)RAND_MAX);
   return rannorm-6;
}

void testregression()
{  // demonstrate least squares estimation by
   // normal equations and by sweep.

   // use normal equations
   int i;
   Matrix x(25,2), y(25,1);
   double betas[2] = {1.0,2.0};
   Matrix beta(2,1,betas);
   for( i=1; i<=25; i++){
     x(i,1) = 1.0;
     x(i,2) = 25-i;
   }
   y = x*beta;
   // throw in some random normal noise
   for( i=1; i<=25; i++) y(i) += z()*0.25;
   Matrix betahat = (x.t()*x).Inv()*x.t()*y;
   cout << betahat;

   // now use sweep
   Matrix xy(25,3);
   for( i=1; i<=25; i++ ) {
     xy(i,1) = x(i,1);
     xy(i,2) = x(i,2);
     xy(i,3) = y(i);
   }
   cout << xy;
   Matrix xypxy = (xy.t()*xy).Sweep(1,2);
   cout << xypxy;
   // betahat is in the first two rows of the last column.
   // residual sum of squares is in 3,3 element of xypxy
   Matrix resid = y - x*betahat;
   cout << (resid.t()*resid);
}
Matrix BadFunc()
{
   Matrix t(1,1,0.0);
   return t;
}

void testthrow()
{
   Matrix t=I(4)+Matrix(4,4,1.0);
   t = t + BadFunc();

}
main()
{

  srand(1234);
  cout << setiosflags(ios::right)
       << setiosflags(ios::fixed)
       << setiosflags(ios::adjustfield)
       << setw(6)
       << setprecision(3)
       << endl;

  // test elementwise operations
  cout << (Matrix(4,3,2.0)
         +Matrix(4,3,1.0)
         -Matrix(4,3,3.0)
         %Matrix(4,3,2.0)
         /Matrix(4,3,2.0));


  // test negation, Identity matrix
  Matrix t = -(I(4)+Matrix(4,4,1.0));

  // test inversion, real, imag
  cout << t << t.Inv();
  cout << t*t.Inv();


  try{
  testthrow();
  }
  catch(MatrixException &e){
    cout << e.msg << endl;
  }
  testregression();


return 0;
}

