//---------------------------------------------------------------------------
//	TLookupDateEdit header file   ( Do Not Edit )
//	 1997 Kenneth Kestner
//---------------------------------------------------------------------------
#ifndef TLookupDateEditH
#define TLookupDateEditH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
#include <vcl\StdCtrls.hpp>
#include <stdlib.h>
#include <vcl\dstring.h>

#include "TFormEnterDate.h"
#include "TAnsiStringTokenizer.h"

//---------------------------------------------------------------------------


class TLookupDateEdit : public TEdit
{
public:
	//Allowable date formats (  MM/DD/YYYY; MM-DD-YYYY;
	// MMMMMMMMMMMMMM DD, YYYY; and MMM DD, YYYY;
	enum FormatType { DF_MMDDYYYY_SLASHES,
		DF_MMDDYYYY_DASHES, DF_FULL, DF_SHORT };

	// C'Tor
	__fastcall 	TLookupDateEdit(TComponent* Owner);

    //Set the displayed date format
	void __fastcall SetDateFormat( enum FormatType dtFormat );

    // Set the date
    bool __fastcall SetDate( TDateTime dateTime = TDateTime::CurrentTime() );

    //Return the date
    TDateTime __fastcall GetDate();

    //Return the date as a string
    AnsiString __fastcall GetDateAsString( enum FormatType dtFormat  );
    AnsiString __fastcall GetDateAsString();

    //Display the date lookup dialog
	void __fastcall DateLookup();

    // Valid (or empty) date == true
    bool __fastcall IsValid();

    // Empty field == true
    bool __fastcall IsEmpty();

    //Return information about this version of the component
    AnsiString __fastcall GetVersionInfo();

    void __fastcall SetDlgCaption( AnsiString theCaption );
    AnsiString __fastcall GetDlgCaption();


private:
    bool	Validate(unsigned short&, unsigned short&,unsigned short&);
    bool	ValidatePartial(unsigned short&, unsigned short&, unsigned short&,
    		unsigned short& );
    bool	CheckDate(unsigned short,unsigned short,unsigned short);
    void	FormatContentsFromField();
    void 	FormatContentsFromDate( TDateTime theDate = TDateTime::CurrentDate());
	unsigned short DetermineMonth( const AnsiString & );
    unsigned short DetermineYear( const AnsiString & );

    bool				m_chimeOnEdit;
    bool				m_focusOnEdit;
    bool				m_modifyYear;
	enum FormatType		m_dateFormat;
    AnsiString			m_dlgCaption;
    bool				m_allowPartialDate;

__published:
	// date format
	__property FormatType DtDateFormat =
    	{read=m_dateFormat,write=SetDateFormat,default=DF_MMDDYYYY_SLASHES};

    // 'chime' or 'beep' if a date edit error occurs
    __property bool DtChimeOnEdit =
    	{read=m_chimeOnEdit,write=m_chimeOnEdit,default=true};

    // set focus on the property after tabbing away if a date
    // edit error occurs
    __property bool DtFocusOnEdit =
    	{read=m_focusOnEdit,write=m_focusOnEdit,default=true};

    // add 1900 to two digit years
    __property bool DtModifyYear =
    	{read=m_modifyYear, write=m_modifyYear, default=true};

    // dialog caption
    __property AnsiString DtDlgCaption =
    	{ read = GetDlgCaption, write = SetDlgCaption};

    //allow partial dates
    __property bool DtAllowPartialDate =
    	{ read = m_allowPartialDate, write = m_allowPartialDate,
        	default = false};

protected:
    virtual void __fastcall DoExit( );

};
//---------------------------------------------------------------------------
#endif
