////////////////////////////////////////////////////////////////////////////////
// TWallPaper component - Oxsoft Heart V5.0
//
// Controls that allows to display a bitmap as a wall paper (like in Windows)
//
// by Alan GARNY
//    gry@physiol.ox.ac.uk
//    http://www.physiol.ox.ac.uk/~gry
//
//  Copyright 1996-97
////////////////////////////////////////////////////////////////////////////////
// Date of Creation: 12/06/97
//
// Modifications: (model: [<initials>, dd/mm/yy] <what has been done>)
////////////////////////////////////////////////////////////////////////////////

#include <vcl\vcl.h>
#pragma hdrstop

////////////////////////////////////////////////////////////////////////////////

#include "Wallpaper.h"

////////////////////////////////////////////////////////////////////////////////
//
// CONSTRUCTOR & DESTRUCTOR
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// TWallPaper: constructor

__fastcall TWallPaper::TWallPaper(TComponent *aOwner)
   : TCustomControl(aOwner)
{
   ControlStyle = TControlStyle() << csAcceptsControls << csCaptureMouse <<
                  csDoubleClicks << csOpaque << csReplicatable;

   Width = Height = 123;

   FBitmap = new Graphics::TBitmap;
}

////////////////////////////////////////////////////////////////////////////////
// ~TWallPaper: destructor

__fastcall TWallPaper::~TWallPaper()
{
   delete FBitmap;
}

////////////////////////////////////////////////////////////////////////////////
//
// METHOD TO DRAW THE CONTROL
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Paint:

void __fastcall TWallPaper::Paint()
{
   if (FBitmap->Width && FBitmap->Height)
   {
      // The painting depends on "FDisplay"

      if (FDisplay == diTile)
      {
/* This soln isn't good enough, cuz the bitmap can be wider/higher than 8 pixels
         Canvas->Brush->Bitmap = new Graphics::TBitmap;

         Canvas->Brush->Bitmap->Assign(FBitmap);

         Canvas->FillRect(GetClientRect());

         delete Canvas->Brush->Bitmap;
*/

         // Do the tiling in a buffer that we copy to the canvas once it's done
         // Reason: we want to avoid too much flickering

         Graphics::TBitmap *Buffer = new Graphics::TBitmap;

         Buffer->Width  = Width;
         Buffer->Height = Height;

         for (int i = 0; i < Width; i += FBitmap->Width)
            for (int j = 0; j < Height; j += FBitmap->Height)
               Buffer->Canvas->Draw(i, j, FBitmap);

         Canvas->CopyRect(Rect(0, 0, Width, Height),
                          Buffer->Canvas,
                          Rect(0, 0, Width, Height));

         delete Buffer;
      }
      else
         Canvas->Draw((Width - FBitmap->Width) / 2,
                      (Height - FBitmap->Height) / 2,
                      FBitmap);
   }
}

////////////////////////////////////////////////////////////////////////////////
//
// METHODS HANDLING DIFFERENT MESSAGES
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// WMSize:

void __fastcall TWallPaper::WMSize(TWMSize &aMsg)
{
   Realign();   // Useful for the children which have the "Align" property
                // different from "alNone"

   Repaint();
}

////////////////////////////////////////////////////////////////////////////////
//
// METHODS TO MODIFY THE DIFFERENT PUBLISHED PROPERTIES
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// SetDisplay: specifies the way the wall paper has to look like

void __fastcall TWallPaper::SetDisplay(const TWallPaperDisplay aValue)
{
   if (aValue != FDisplay)
   {
      FDisplay = aValue;

      Repaint();
   }
}

////////////////////////////////////////////////////////////////////////////////
// SetBitmap: loads a bitmap

void __fastcall TWallPaper::SetBitmap(Graphics::TBitmap *aValue)
{
   FBitmap->Assign(aValue);

   Repaint();
}

////////////////////////////////////////////////////////////////////////////////
// Registration

namespace Wallpaper
{
   void __fastcall Register()
   {
      TComponentClass classes[1] = {__classid(TWallPaper)};

      RegisterComponents("GRY", classes, 0);
   }
}

// End of file
////////////////////////////////////////////////////////////////////////////////

